#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Base item information
ITEM_DATAFLAG_BASE = 0x00000001
# Item custom properties
ITEM_DATAFLAG_CUSTOM_PROPS = 0x00000002
# Item billing/construction properties
ITEM_DATAFLAG_BILLING_PROPS = 0x00000004
# Item custom fields
ITEM_DATAFLAG_CUSTOM_FIELDS = 0x00000008
# Item image 
ITEM_DATAFLAG_IMAGE = 0x00000010
# Item messages
ITEM_DATAFLAG_MESSAGES = 0x00000020
# Item GUID
ITEM_DATAFLAG_GUID = 0x00000040
# Admin fields plugin
ITEM_DATAFLAG_ADMINFIELDS = 0x00000080

# User has at least view access to given item
ITEM_ACCESSFLAG_VIEW = 0x1
# User can view detailed item properties
ITEM_ACCESSFLAG_VIEW_PROPERTIES = 0x2
# User can update ACL settings for item or user
ITEM_ACCESSFLAG_SET_ACL = 0x4
# User can delete item
ITEM_ACCESSFLAG_DELETE_ITEM = 0x8
# User can change item name
ITEM_ACCESSFLAG_EDIT_NAME = 0x10
# view custom fields
ITEM_ACCESSFLAG_VIEW_CFIELDS = 0x20
# create, edit and delete custom fields
ITEM_ACCESSFLAG_EDIT_CFIELDS = 0x40
# edit other item settings, not mentioned with own mask
ITEM_ACCESSFLAG_EDIT_OTHER = 0x80
# edit item image
ITEM_ACCESSFLAG_EDIT_IMAGE = 0x100
# execute reports over item, incl. raw messages access
ITEM_ACCESSFLAG_EXEC_REPORTS = 0x200
# edit ACL-propagated items, e.g. edit units in units group
ITEM_ACCESSFLAG_EDIT_SUBITEMS = 0x400
# view, add and delete item log records
ITEM_ACCESSFLAG_MANAGE_LOG = 0x800
# view admin fields
ITEM_ACCESSFLAG_VIEW_ADMINFIELDS = 0x1000
# create, edit and delete admin fields
ITEM_ACCESSFLAG_EDIT_ADMINFIELDS = 0x2000

# AVL message flags mask determining message type
ITEM_MESSAGEFLAG_TYPE_MASK = 0xFF00
# AVL message flag, set when message represents data sent from unit
ITEM_MESSAGEFLAG_TYPE_UNIT_DATA = 0x0000
# AVL message flag, set when message represents incoming SMS
ITEM_MESSAGEFLAG_TYPE_UNIT_SMS = 0x0100
# AVL message flag, set when message represents command over unit
ITEM_MESSAGEFLAG_TYPE_UNIT_CMD = 0x0200
#  AVL message flag, set when message represents event
ITEM_MESSAGEFLAG_TYPE_UNIT_EVENT = 0x0600
# Storage message flag, set when message represents log of user actions
ITEM_MESSAGEFLAG_TYPE_USER_LOG = 0x0400
# AVL message flag, set when message type is user notification
ITEM_MESSAGEFLAG_TYPE_NOTIFICATION = 0x0300
# Billing message flag, set when message represents billing balance update
ITEM_MESSAGEFLAG_TYPE_BALANCE = 0x0500
# Storage message flag, set when message represents plot cultivation
ITEM_MESSAGEFLAG_TYPE_AGRO_CULTIVATION = 0x0700
# AVL message type, set when message type is SMS from driver
ITEM_MESSAGEFLAG_TYPE_DRIVER_SMS = 0x0900
# AVL message flag, set when represents unit log messages
ITEM_MESSAGEFLAG_TYPE_LOG_RECORD = 0x1000
# AVL message flag, set when message type is something different
ITEM_MESSAGEFLAG_TYPE_OTHER = 0xFF00

""" Data flags constants """
# Drivers plugin
ITEM_RESOURCE_DATAFLAG_DRIVERS = 0x00000100
# Jobs plugin
ITEM_RESOURCE_DATAFLAG_JOBS = 0x00000200
# POI plugin
ITEM_RESOURCE_DATAFLAG_POI = 0x00000800
# Notifications plugin
ITEM_RESOURCE_DATAFLAG_NOTIFICATIONS = 0x00000400
# Geofences plugin
ITEM_RESOURCE_DATAFLAG_ZONES = 0x00001000
# Reports plugin
ITEM_RESOURCE_DATAFLAG_REPORTS = 0x00002000
# Agro plugins
ITEM_RESOURCE_DATAFLAG_AGRO = 0x01000000
# Driver units
ITEM_RESOURCE_DATAFLAG_DRIVER_UNITS = 0x00004000
# Driver groups plugin
ITEM_RESOURCE_DATAFLAG_DRIVER_GROUPS = 0x00008000
# Trailers plugin
ITEM_RESOURCE_DATAFLAG_TRAILERS = 0x00010000
# Trailer groups plugin
ITEM_RESOURCE_DATAFLAG_TRAILER_GROUPS = 0x00020000
# Trailer units
ITEM_RESOURCE_DATAFLAG_TRAILER_UNITS = 0x00040000

""" Unit dataflgs """
# Unit restricted props
ITEM_UNIT_DATAFLAG_RESTRICTED = 0x00000100
# Unit commands
ITEM_UNIT_DATAFLAG_COMMANDS = 0x00000200
# Unit commands aliases
ITEM_UNIT_DATAFLAG_COMMAND_ALIASES = 0x00080000
# TODO

""" User flags """
#
ITEM_USER_USERFLAG_CAN_CREATE_ITEMS = 0x00000004
# TODO

""" Item ACL flags """
# User has at least view access to given item
ITEM_ACCESSFLAG_VIEW = 0x1

""" Resource ACL flags """
# View notification
ITEM_RESOURCE_ACCESSFLAG_VIEW_NOTIFICATION = 0x100000
# Create, edit and delete notification
ITEM_RESOURCE_ACCEESSFLAG_EDIT_NOTIFICATION = 0x200000
# View jobs
ITEM_RESOURCE_ACCEESSFLAG_VIEW_JOBS = 0x4000000
# Edit jobs
ITEM_RESOURCE_ACCEESSFLAG_EDIT_JOBS = 0x8000000
# View POI
ITEM_RESOURCE_ACCEESSFLAG_VIEW_POI = 0x400000
# Edit POI
ITEM_RESOURCE_ACCEESSFLAG_EDIT_POI = 0x800000
# View geozones
ITEM_RESOURCE_ACCEESSFLAG_VIEW_ZONES = 0x1000000
# Edit geozones
ITEM_RESOURCE_ACCEESSFLAG_EDIT_ZONES = 0x2000000
# View reports
ITEM_RESOURCE_ACCEESSFLAG_VIEW_REPORTS = 0x10000000

""" Unit ACL flags """
# View, edit connectivity settings: HW, UID, Phone, Password, input-message-filter
ITEM_UNIT_ACCESSFLAG_EDIT_DEVICE = 0x100000
# Execute commands over unit
ITEM_UNIT_ACCESSFLAG_EXECUTE_COMMANDS = 0x1000000
# View command aliases
ITEM_UNIT_ACCESSFLAG_VIEW_COMMAND_ALIASES = 0x400000000
# Include unit in various auto-processing systems like retranslation, jobs, notifications
ITEM_UNIT_ACCESSFLAG_MONITOR_STATE = 0x8000000000
# Register various unit events, store counters, bind/unbind drivers and change unit status
ITEM_UNIT_ACCESSFLAG_REGISTER_EVENTS = 0x2000000



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from . import flags

from .api import Wialon, WialonError
# Silence potential warnings from static analysis tools:
assert Wialon
assert WialonError

__author__ = "Alex Chernetsky chal@gurtam.com"
__copyright__ = ("Copyright 2013-2016, Gurtam; ",)

__credits__ = ["Alex Chernetsky", "Aleksey Shmigelski", "Mike Turchunovich"]
__version__ = "1.1.6"

__all__ = ["Wialon"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from builtins import str

try:
    from urllib import urlencode
    from urlparse import urljoin
except Exception:
    from urllib.parse import urlencode, urljoin

try:
    from urllib2 import Request, urlopen, HTTPError, URLError
except ImportError:
    from urllib.request import Request, urlopen
    from urllib.error import HTTPError, URLError

try:
    import simplejson as json
    assert json  # Silence potential warnings from static analysis tools
except ImportError:
    import json


class WialonError(Exception):
    """
    Exception raised when an Wialon Remote API call fails due to a network
    related error or for a Wialon specific reason.
    """
    errors = {
        1: 'Invalid session',
        2: 'Invalid service',
        3: 'Invalid result',
        4: 'Invalid input',
        5: 'Error performing request',
        6: 'Unknow error',
        7: 'Access denied',
        8: 'Invalid user name or password',
        9: 'Authorization server is unavailable, please try again later',
        1001: 'No message for selected interval',
        1002: 'Item with such unique property already exists',
        1003: 'Only one request of given time is allowed at the moment'
    }

    def __init__(self, code, text):
        self._text = text
        self._code = code
        try:
            self._code = int(code)
        except ValueError:
            pass

    def __unicode__(self):
        explanation = self._text
        if (self._code in WialonError.errors):
            explanation = " ".join([WialonError.errors[self._code], self._text])

        message = u'{error} ({code})'.format(error=explanation, code=self._code)
        return u'WialonError({message})'.format(message=message)

    def __str__(self):
        return self.__unicode__()

    def __repr__(self):
        return str(self)


class Wialon(object):
    def __init__(self, scheme='http',  host="hst-api.wialon.com", port=80, sid=None, **extra_params):
        """
        Created the Wialon API object.
        """
        self._sid = sid
        self.__default_params = {}
        self.__default_params.update(extra_params)

        self.__base_url = (
            '{scheme}://{host}:{port}'.format(
                scheme=scheme,
                host=host,
                port=port
            )
        )

        self.__base_api_url = urljoin(self.__base_url, 'wialon/ajax.html?')

    @property
    def sid(self):
        return self._sid

    @sid.setter
    def sid(self, value):
        self._sid = value

    def update_extra_params(self, **params):
        """
        Updated the Wialon API default parameters.
        """
        self.__default_params.update(params)

    def avl_evts(self):
        """
        Call avl_event request
        """
        url = urljoin(self.__base_url, 'avl_evts')
        params = {
            'sid': self.sid
        }

        return self.request('avl_evts', url, params)

    def call(self, action, *argc, **kwargs):
        """
        Call the API method provided with the parameters supplied.
        """

        if (not kwargs):
            # List params for batch
            if isinstance(argc, tuple) and len(argc) == 1:
                params = json.dumps(argc[0], ensure_ascii=False)
            else:
                params = json.dumps(argc, ensure_ascii=False)
        else:
            params = json.dumps(kwargs, ensure_ascii=False)

        params = {
            'svc': action.replace('_', '/', 1),
            'params': params.encode("utf-8"),
            'sid': self.sid
        }

        all_params = self.__default_params.copy()
        all_params.update(params)
        return self.request(action, self.__base_api_url, all_params)

    def request(self, action, url, params):
        url_params = urlencode(params)
        data = url_params.encode('utf-8')
        try:
            request = Request(url, data)
            response = urlopen(request)
            response_content = response.read()
        except HTTPError as e:
            raise WialonError(0, u"HTTP {code}".format(code=e.code))
        except URLError as e:
            raise WialonError(0, str(e))

        content_type = response.info().get('Content-Type')
        result = response_content
        try:
            if content_type == 'application/json':
                result = result.decode('utf-8', errors='ignore')
                result = json.loads(result)
        except ValueError as e:
            raise WialonError(
                0,
                u"Invalid response from Wialon: {0}".format(e),
            )

        if (isinstance(result, dict) and 'error' in result and result['error'] > 0):
            raise WialonError(result['error'], action)

        errors = []
        if isinstance(result, list):
            # Check for batch errors
            for elem in result:
                if (not isinstance(elem, dict)):
                    continue
                if "error" in elem:
                    errors.append("%s (%d)" % (WialonError.errors[elem["error"]], elem["error"]))

        if (errors):
            errors.append(action)
            raise WialonError(0, " ".join(errors))

        return result

    def __getattr__(self, action_name):
        """
        Enable the calling of Wialon API methods through Python method calls
        of the same name.
        """
        def get(self, *args, **kwargs):
            return self.call(action_name, *args, **kwargs)

        return get.__get__(self)

if __name__ == '__main__':
    try:
        wialon_api = Wialon()
        # token/login request
        token = 'TEST TOKEN HERE'
        result = wialon_api.token_login(token=token)
        wialon_api.sid = result['eid']
        # get events
        result = wialon_api.avl_evts()
        # core/logout request
        wialon_api.core_logout()
    except WialonError:
        pass



#!/usr/bin/env python
import sys
import wialon

from setuptools import setup, find_packages


extra = {}
if sys.version_info >= (3,):
    extra['use_2to3'] = True
    extra['convert_2to3_doctests'] = ['README.md']

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Software Development :: Libraries :: Python Modules'
    ]

KEYWORDS = 'wialon remote api wrapper'

setup(name='python-wialon',
      version=wialon.__version__,
      description="""Wialon Remote API wrapper for Python.""",
      long_description=open('README.md').read(),
      long_description_content_type="text/markdown",
      author=wialon.__author__,
      url='https://github.com/wialon/python-wialon',
      packages=find_packages(),
      download_url='http://pypi.python.org/pypi/python-wialon/',
      classifiers=CLASSIFIERS,
      keywords=KEYWORDS,
      zip_safe=True,
      install_requires=['simplejson', 'future'],
      py_modules=['python-wialon']
)
